'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ParseCliController = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _cache = require('parse-server/lib/cache');

var _cache2 = _interopRequireDefault(_cache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DeployInfoBasePath = "_deployInfo";
var DeployInfoCollectionName = "deployInfo";
var FilesCollectionName = "deployFile";

function computeChecksum(content) {
  var algo = 'md5',
      md5sum = _crypto2.default.createHash(algo);
  md5sum.update(content, 'binary');
  return md5sum.digest('hex');
}

var ParseCliController = function () {
  function ParseCliController(vendorAdapter) {
    _classCallCheck(this, ParseCliController);

    this.vendorAdapter = vendorAdapter;
  }

  _createClass(ParseCliController, [{
    key: 'getCliLatestVersion',
    value: function getCliLatestVersion() {
      return this.vendorAdapter.getCliLatestVersion();
    }
  }, {
    key: 'isSupported',
    value: function isSupported(mode, other, version) {
      return new Promise(function (resolve, reject) {
        // local commands must be defined
        var localCommands = ["default", "generate", "-h", "--help", "help", "migrate", "update", "version"],
            commands = ["add", "configure accountkey", "deploy", "develop", "download", "functions", "jssdk", "list", "logs", "new", "releases", "rollback", "triggers", "cloud"],
            nokCommands = ["symbols"],
            tokens = {};
        localCommands.forEach(function (value) {
          tokens[value] = true;
        });
        commands.forEach(function (value) {
          tokens[value] = true;
        });

        if (mode == 'parse') {
          var parts = other.split(" ");
          for (var i = 1; i <= parts.length; i++) {
            var token = parts.slice(0, i).join(" ");
            if (tokens[token]) {
              resolve({});
              return;
            }
          }
        }
        reject();
      });
    }
  }, {
    key: 'getAccountKey',
    value: function getAccountKey(email, password) {
      return this.vendorAdapter.getAccountKey(email, password);
    }
  }, {
    key: 'getEmail',
    value: function getEmail(accountKey) {
      return this.vendorAdapter.getEmail(accountKey);
    }
  }, {
    key: 'getJsVersions',
    value: function getJsVersions(appId) {
      return this.vendorAdapter.getJsVersions(appId);
    }
  }, {
    key: 'getApps',
    value: function getApps(accountKey) {
      return this.vendorAdapter.getApps(accountKey);
    }
  }, {
    key: 'getApp',
    value: function getApp(accountKey, appId) {
      return this.vendorAdapter.getApp(accountKey, appId);
    }
  }, {
    key: 'createApp',
    value: function createApp(accountKey, appName) {
      return this.vendorAdapter.createApp(accountKey, appName);
    }
  }, {
    key: 'getFile',
    value: function getFile(appId, folder, filename, version, checksum) {
      var config = _cache2.default.get(appId),
          query = { folder: folder, filename: filename, version: version, checksum: checksum };
      return config.databaseController.find(this.getCollectionName(appId, FilesCollectionName), query).then(function (objects) {
        if (objects.length == 0) {
          throw new Error("Object " + JSON.stringify(query) + " not found.");
        }
        var object = objects[0];
        return config.filesController.getFileData(config, object.name).then(function (data) {
          // Data is base64 encoded
          if (object.encoding == 'base64') {
            data = new Buffer(data.toString(), 'base64');
          }
          return data;
        });
      });
    }
  }, {
    key: 'uploadFile',
    value: function uploadFile(appId, folder, filename, content) {
      var _this = this;

      var config = _cache2.default.get(appId),
          key = _path2.default.join(folder, filename),
          binaryContent = new Buffer(content, 'base64').toString('binary'),
          checksum = computeChecksum(binaryContent),
          filesCollectionName = this.getCollectionName(appId, FilesCollectionName);

      return config.databaseController.find(filesCollectionName, {
        folder: folder,
        filename: filename
      }, {
        sort: { createdAt: 1 }
      }).then(function (objects) {
        var nextVersion;

        if (objects.length == 0) {
          nextVersion = 1;
        } else {
          var object = objects[0],
              currentVersion = object.version;
          nextVersion = parseInt(currentVersion) + 1;
        }
        return nextVersion.toString();
      }).then(function (version) {
        return config.filesController.createFile(config, _path2.default.join(_this.getBasePath(appId, DeployInfoBasePath), key), content).then(function (obj) {
          return config.databaseController.create(filesCollectionName, {
            version: version,
            checksum: checksum,
            folder: folder,
            filename: filename,
            name: obj.name,
            url: obj.url,
            encoding: 'base64'
          });
        }).then(function () {
          return {
            checksum: checksum,
            version: version
          };
        });
      });
    }
  }, {
    key: '_findDeployInfo',
    value: function _findDeployInfo(appId, query, sort) {
      var _this2 = this;

      var config = _cache2.default.get(appId);
      var options = {
        sort: sort,
        limit: 1
      };
      return config.databaseController.find(this.getCollectionName(appId, DeployInfoCollectionName), query, options).then(function (results) {
        return results.map(function (deployInfo) {
          if (_this2.vendorAdapter.filterFiles) {
            _this2.vendorAdapter.filterFiles(deployInfo);
          }
          return _this2._unpatchDeployInfo(deployInfo);
        });
      }).then(function (results) {
        return results.length !== 0 ? results[0] : null;
      });
    }
  }, {
    key: 'getDeployInfo',
    value: function getDeployInfo(appId) {
      return this._findDeployInfo(appId, {}, { deployedAt: -1, createdAt: -1 });
    }
  }, {
    key: '_setDeployInfo',
    value: function _setDeployInfo(appId, deployInfo) {
      var config = _cache2.default.get(appId);
      var className = this.getCollectionName(appId, DeployInfoCollectionName);
      var query = { releaseId: deployInfo.releaseId };
      var now = new Date().toISOString();
      if (deployInfo.createdAt) {
        return config.databaseController.update(className, query, { deployedAt: now });
      } else {
        deployInfo.deployedAt = deployInfo.createdAt = now;

        return config.databaseController.create(className, this._patchDeployInfo(deployInfo));
      }
    }

    // Patch deployInfo to avoid keys invalid chars, like dots in filenames.

  }, {
    key: '_patchDeployInfo',
    value: function _patchDeployInfo(deployInfo) {
      deployInfo.files = [];
      Object.keys(deployInfo.userFiles).forEach(function (folder) {
        Object.keys(deployInfo.userFiles[folder]).forEach(function (filename) {
          var version = deployInfo.userFiles[folder][filename],
              checksum = deployInfo.checksums[folder][filename];
          deployInfo.files.push({
            folder: folder,
            filename: filename,
            version: version,
            checksum: checksum
          });
        });
      });
      delete deployInfo.userFiles;
      delete deployInfo.checksums;
      return deployInfo;
    }
  }, {
    key: '_unpatchDeployInfo',
    value: function _unpatchDeployInfo(deployInfo) {
      deployInfo.userFiles = {};
      deployInfo.checksums = {};

      deployInfo.files.forEach(function (obj) {
        deployInfo.userFiles[obj.folder] = deployInfo.userFiles[obj.folder] || {};
        deployInfo.userFiles[obj.folder][obj.filename] = obj.version;

        deployInfo.checksums[obj.folder] = deployInfo.checksums[obj.folder] || {};
        deployInfo.checksums[obj.folder][obj.filename] = obj.checksum;
      });
      delete deployInfo.files;
      return deployInfo;
    }
  }, {
    key: '_collect',
    value: function _collect(appId, deployInfo, folder) {
      var _this3 = this;

      var promises = Object.keys(deployInfo.userFiles[folder] || {}).map(function (filename) {
        var version = deployInfo.userFiles[folder][filename],
            checksum = deployInfo.checksums[folder][filename];
        return _this3.getFile(appId, folder, filename, version, checksum).then(function (data) {
          return _this3.vendorAdapter.collect(appId, deployInfo, folder, filename, data);
        });
      });

      return Promise.all(promises);
    }
  }, {
    key: 'deploy',
    value: function deploy(appId, deployInfo) {
      var _this4 = this;

      return this._processRollback(appId, deployInfo).then(function (di) {
        deployInfo = di;
        return _this4._processFiles(appId, di);
      }).then(function () {
        deployInfo._checksum = computeChecksum(JSON.stringify(deployInfo));
        return deployInfo;
      }).then(function (di) {
        return _this4._setReleaseMetadata(appId, di);
      }).then(function () {
        return _this4._comparePackageVersion(appId, deployInfo);
      }).then(function (packageInfo) {
        deployInfo.packageChanged = packageInfo.packageChanged;
      }).then(function () {
        return _this4._collect(appId, deployInfo, 'cloud');
      }).then(function () {
        return _this4._collect(appId, deployInfo, 'public');
      }).then(function () {
        return _this4.vendorAdapter.publish(appId, deployInfo);
      }).then(function () {
        delete deployInfo._checksum;
        delete deployInfo.packageChanged;
        return deployInfo;
      }).then(function () {
        return _this4._setDeployInfo(appId, deployInfo);
      }).then(function () {
        return deployInfo;
      });
    }
  }, {
    key: '_getNextReleaseId',
    value: function _getNextReleaseId(appId) {
      return this._findDeployInfo(appId, {}, { releaseId: -1 }).then(function (deployInfo) {
        var currentReleaseId = deployInfo ? deployInfo.releaseId : 0;
        return parseInt(currentReleaseId) + 1;
      });
    }
  }, {
    key: '_setReleaseMetadata',
    value: function _setReleaseMetadata(appId, deployInfo) {
      if (deployInfo.releaseId) {
        return Promise.resolve(deployInfo);
      }
      return this._getNextReleaseId(appId).then(function (deployInfoId) {
        deployInfo.releaseId = deployInfoId;
        deployInfo.releaseName = "v" + deployInfoId;
        return deployInfo;
      });
    }
  }, {
    key: '_processRollback',
    value: function _processRollback(appId, deployInfo) {
      var _this5 = this;

      if (deployInfo.releaseName === undefined) {
        // it is not a rollback
        return Promise.resolve(deployInfo);
      } else if (deployInfo.releaseName === "") {
        // rollback to deploy before the current one
        return this.getDeployInfo(appId).then(function (deployInfo) {
          return _this5._findDeployInfo(appId, { releaseId: { "$lt": deployInfo.releaseId } }, { releaseId: -1 });
        });
      } else {
        return this._findDeployInfo(appId, { releaseName: deployInfo.releaseName }, { deployedAt: -1 });
      }
    }
  }, {
    key: '_processFiles',
    value: function _processFiles(appId, deployInfo) {
      var _this6 = this;

      if (!deployInfo.files) {
        return Promise.resolve(deployInfo);
      }
      if (deployInfo.checksums || deployInfo.userFiles) {
        throw new Error("Files must not be defined with checksums and userFiles.");
      }

      // Prevent deploy request error from dashboard.
      if (deployInfo.checksums === undefined) {
        deployInfo.checksums = {};
      }
      if (deployInfo.userFiles === undefined) {
        deployInfo.userFiles = {};
      }

      var promises = Object.keys(deployInfo.files).map(function (folder) {
        var folderPromises = Object.keys(deployInfo.files[folder] || {}).map(function (filename) {
          var content = deployInfo.files[folder][filename];

          return _this6.uploadFile(appId, folder, filename, content).then(function (obj) {
            deployInfo.checksums[folder] = deployInfo.checksums[folder] || {};
            deployInfo.checksums[folder][filename] = obj.checksum;
            deployInfo.userFiles[folder] = deployInfo.userFiles[folder] || {};
            deployInfo.userFiles[folder][filename] = obj.version;
          });
        });
        return Promise.all(folderPromises);
      });
      return Promise.all(promises).then(function () {
        delete deployInfo.files;
        return deployInfo;
      });
    }
  }, {
    key: '_comparePackageVersion',
    value: function _comparePackageVersion(appId, deployInfo) {
      var _this7 = this;

      var packageInfo = {};
      var promise = deployInfo ? Promise.resolve(deployInfo) : this.getDeployInfo(appId);
      return promise.then(function (deployInfo) {
        return _this7._findDeployInfo(appId, { releaseId: { "$lt": deployInfo.releaseId } }, { releaseId: -1 });
      }).then(function (oldDeployInfo) {
        return _this7._processFiles(appId, oldDeployInfo || {});
      }).then(function (oldDeployInfo) {
        packageInfo.packageChanged = !oldDeployInfo.checksums && deployInfo.checksums && deployInfo.checksums['cloud'] && deployInfo.checksums['cloud']['package.json'] || deployInfo.checksums && oldDeployInfo.checksums && deployInfo.checksums['cloud'] && oldDeployInfo.checksums['cloud'] && deployInfo.checksums['cloud']['package.json'] !== oldDeployInfo.checksums['cloud']['package.json'];
        return packageInfo;
      }).then(function () {
        if (_this7.vendorAdapter.installModules && packageInfo.packageChanged) {
          return _this7.vendorAdapter.installModules(appId).then(function () {
            return packageInfo;
          });
        } else {
          packageInfo.packageChanged = false;
          return packageInfo;
        }
      });
    }
  }, {
    key: 'getCollectionName',
    value: function getCollectionName(appId, collectionName) {
      return this.vendorAdapter.getCollectionName ? this.vendorAdapter.getCollectionName(appId, collectionName) : collectionName;
    }
  }, {
    key: 'getBasePath',
    value: function getBasePath(appId, basePath) {
      return this.vendorAdapter.getBasePath ? this.vendorAdapter.getBasePath(appId, basePath) : basePath;
    }
  }, {
    key: 'getReleases',
    value: function getReleases(appId) {
      var _this8 = this;

      var config = _cache2.default.get(appId);
      return config.databaseController.find(this.getCollectionName(appId, DeployInfoCollectionName), {}, {
        sort: { createdAt: 1 }
      }).then(function (results) {
        return results.map(function (deployInfo) {
          return _this8._unpatchDeployInfo(deployInfo);
        });
      }).then(function (results) {
        return results.map(function (deployInfo) {
          return {
            version: deployInfo.releaseName,
            description: deployInfo.description,
            timestamp: deployInfo.createdAt,
            // parse-cli expects a string for no reason
            userFiles: JSON.stringify(deployInfo.userFiles)
          };
        });
      });
    }
  }]);

  return ParseCliController;
}();

exports.default = ParseCliController;
exports.ParseCliController = ParseCliController;