'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ParseCliRouter = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _middlewares = require('parse-server/lib/middlewares');

var middlewares = _interopRequireWildcard(_middlewares);

var _PromiseRouter2 = require('parse-server/lib/PromiseRouter');

var _PromiseRouter3 = _interopRequireDefault(_PromiseRouter2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var promiseHandleParseHeaders = function promiseHandleParseHeaders(req) {
  return new Promise(function (resolve, reject) {
    var error = new Error();
    var resStub = {
      status: function status(statusCode) {
        error.status = statusCode;
      },
      end: function end(message) {
        error.message = JSON.parse(message).error;
      }
    };
    var next = function next() {
      resolve();
    };
    var response = middlewares.handleParseHeaders(req, resStub, next);
    if (error.status) {
      throw error;
    }
    return response;
  });
};

var ParseCliRouter = function (_PromiseRouter) {
  _inherits(ParseCliRouter, _PromiseRouter);

  function ParseCliRouter(controller) {
    _classCallCheck(this, ParseCliRouter);

    var _this = _possibleConstructorReturn(this, (ParseCliRouter.__proto__ || Object.getPrototypeOf(ParseCliRouter)).call(this));

    _this.controller = controller;
    return _this;
  }

  _createClass(ParseCliRouter, [{
    key: 'promiseEnforceAccountKeyAccess',
    value: function promiseEnforceAccountKeyAccess(req) {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        var accountKey = req.get('X-Parse-Account-Key');
        if (accountKey) {
          return resolve(accountKey);
        } else {
          var email = req.get('X-Parse-Email');
          var password = req.get('X-Parse-Password');
          return _this2.controller.getAccountKey(email, password).then(function (accountKey) {
            if (accountKey) {
              return resolve(accountKey);
            } else {
              return reject("invalid email and password");
            }
          });
        }
      }).then(function (accountKey) {
        if (!req.config) {
          req.config = {};
        }
        req.config.accountKey = accountKey;
      }).catch(function (error) {
        var errorObj = new Error();
        if (error === 'invalid email and password') {
          errorObj.status = 401;
        } else {
          errorObj.status = 400;
        }
        errorObj.message = error;

        throw errorObj;
      });
    }

    /*
    Before execute any command, parse-cli check with the server
    if it is supported.
     Arguments are passed by query string.
     mode="parse"
    other=<command arguments>
    version=<parse-cli version>
    */

  }, {
    key: 'isSupported',
    value: function isSupported(req) {
      var mode = req.query.mode,
          other = req.query.other,
          version = req.query.version;

      if (version == 'latest') {
        return this.controller.getCliLatestVersion().then(function (latestVersion) {
          return {
            response: {
              version: latestVersion
            }
          };
        });
      }

      return this.controller.isSupported(mode, other, version).then(function (_ref) {
        var warning = _ref.warning;

        var obj = {};
        if (warning) {
          obj.warning = warning;
        }
        return {
          response: obj
        };
      }, function (err) {
        console.log(err);
        var error = { 'error': 'unsupported method: ' + other };
        return {
          status: 400,
          response: error
        };
      });
    }
  }, {
    key: 'configureAccountKey',
    value: function configureAccountKey(req) {
      var accountKey = req.body.accountKey;

      return this.controller.getEmail(accountKey).then(function (email) {
        return {
          response: { email: email }
        };
      }, function () {
        return {
          status: 400,
          response: {
            error: 'accountKey not found: ' + accountKey
          }
        };
      });
    }
  }, {
    key: 'getAccountKey',
    value: function getAccountKey(req) {
      var password = req.get('X-Parse-Password'),
          email = req.get('X-Parse-Email');

      this.controller.getAccountKey(email, password).then(function (key) {
        return {
          status: 200,
          response: {
            accountKey: key
          }
        };
      });
    }
  }, {
    key: 'getJsVersions',
    value: function getJsVersions(req) {
      return this.controller.getJsVersions(req.config.applicationId).then(function (versions) {
        return {
          response: {
            js: versions
          }
        };
      });
    }
  }, {
    key: 'getApps',
    value: function getApps(req) {
      return this.controller.getApps(req.config.accountKey).then(function (parseApps) {
        return {
          response: {
            results: parseApps
          }
        };
      });
    }
  }, {
    key: 'getApp',
    value: function getApp(req) {
      var applicationId = req.params.applicationId;
      return this.controller.getApp(req.config.accountKey, applicationId).then(function (parseApp) {
        return {
          response: parseApp
        };
      });
    }
  }, {
    key: 'createApp',
    value: function createApp(req) {
      var appName = req.body.appName;
      return this.controller.createApp(req.config.accountKey, appName).then(function (parseApp) {
        return { response: parseApp };
      });
    }

    /*
    uploadFile body example:
     {
      "name" : "index.html",
      "content": <base64 file content>
    }
    */

  }, {
    key: 'uploadFile',
    value: function uploadFile(req, folder) {
      var filename = req.body.name,
          content = req.body.content,
          contentType = req.headers['content-type'];

      return this.controller.uploadFile(req.config.applicationId, folder, filename, content, contentType).then(function (obj) {
        return {
          response: {
            version: obj.version
          }
        };
      });
    }
  }, {
    key: 'getFile',
    value: function getFile(req, folder, options) {
      var options = options || {},
          version = req.query.version,
          filename,
          checksum = req.query.checksum;

      if (folder === 'cloud') filename = /\/scripts\/(.+)\?checksum/.exec(req.url)[1];else filename = /\/hosted_files\/(.+)\?checksum/.exec(req.url)[1];

      filename = decodeURI(filename);

      return this.controller.getFile(req.config.applicationId, folder, filename, version, checksum).then(function (data) {
        if (options.base64) {
          data = data.toString('base64');
        } else {
          data = data.toString();
        }
        return {
          response: data
        };
      });
    }
  }, {
    key: 'getDeployInfo',
    value: function getDeployInfo(req) {
      return this.controller.getDeployInfo(req.config.applicationId).then(function (deployInfo) {
        deployInfo = deployInfo || {};
        return {
          response: deployInfo
        };
      });
    }

    /*
    Deploy body example:
     {
      "parseVersion" : "1.9.2",
      "checksums": {
        "cloud": {
          "main.js": "0e50654c95071583dcf707dda75ee6cc"
        },
        "public": {
          "index.js": "a9d71ca772c4fb43973b93322f3c39a5"
        }
      },
      "userFiles": {
        "cloud": {
          "main.js": "1"
        },
        "public": {
          "index.js": "2"
        }
      }
    }
    */

  }, {
    key: 'deploy',
    value: function deploy(req) {
      return this.controller.deploy(req.config.applicationId, req.body).then(function (deployInfo) {
        return {
          response: deployInfo
        };
      });
    }
  }, {
    key: 'getReleases',
    value: function getReleases(req) {
      return this.controller.getReleases(req.config.applicationId).then(function (releases) {
        return {
          response: releases
        };
      });
    }
  }, {
    key: 'symbolFiles',
    value: function symbolFiles(req) {
      // TODO: understand what this command should do
      throw new Error("symbols upload is not implemented.");
    }
  }, {
    key: 'mountRoutes',
    value: function mountRoutes() {
      var _this3 = this;

      this.route('GET', '/supported', function (req) {
        return _this3.isSupported(req);
      });
      this.route('GET', 'accountKey', function (req) {
        return _this3.getAccountKey(req);
      });
      this.route('POST', '/accountKey', function (req) {
        return _this3.configureAccountKey(req);
      });
      this.route('GET', '/jsVersions', promiseHandleParseHeaders, middlewares.promiseEnforceMasterKeyAccess, function (req) {
        return _this3.getJsVersions(req);
      });

      // TODO: validate Parse-Account-Key
      this.route('GET', '/apps', function (req) {
        return _this3.promiseEnforceAccountKeyAccess(req);
      }, function (req) {
        return _this3.getApps(req);
      });
      this.route('GET', '/apps/:applicationId', function (req) {
        return _this3.promiseEnforceAccountKeyAccess(req);
      }, function (req) {
        return _this3.getApp(req);
      });
      this.route('POST', '/apps', function (req) {
        return _this3.promiseEnforceAccountKeyAccess(req);
      }, function (req) {
        return _this3.createApp(req);
      });

      this.route('POST', '/scripts', promiseHandleParseHeaders, middlewares.promiseEnforceMasterKeyAccess, function (req) {
        return _this3.uploadFile(req, 'cloud');
      });
      this.route('GET', '/scripts/*', promiseHandleParseHeaders, middlewares.promiseEnforceMasterKeyAccess, function (req) {
        return _this3.getFile(req, 'cloud');
      });
      this.route('POST', '/hosted_files', promiseHandleParseHeaders, middlewares.promiseEnforceMasterKeyAccess, function (req) {
        return _this3.uploadFile(req, 'public');
      });
      this.route('GET', '/hosted_files/*', promiseHandleParseHeaders, middlewares.promiseEnforceMasterKeyAccess,
      // public folder must be base64 encoded
      function (req) {
        return _this3.getFile(req, 'public', { base64: true });
      });

      this.route('GET', '/deploy', promiseHandleParseHeaders, middlewares.promiseEnforceMasterKeyAccess, function (req) {
        return _this3.getDeployInfo(req);
      });
      this.route('POST', '/deploy', promiseHandleParseHeaders, middlewares.promiseEnforceMasterKeyAccess, function (req) {
        return _this3.deploy(req);
      });

      this.route('GET', '/releases', promiseHandleParseHeaders, middlewares.promiseEnforceMasterKeyAccess, function (req) {
        return _this3.getReleases(req);
      });

      this.route('POST', '/symbolFiles/:name', function (req) {
        return _this3.symbolFiles(req);
      });
    }
  }]);

  return ParseCliRouter;
}(_PromiseRouter3.default);

exports.default = ParseCliRouter;
exports.ParseCliRouter = ParseCliRouter;