'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.VendorAdapter = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _package = require('parse-server/package.json');

var _triggers = require('parse-server/lib/triggers');

var triggers = _interopRequireWildcard(_triggers);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var VendorAdapter = function () {
  function VendorAdapter(_ref) {
    var config = _ref.config,
        cloud = _ref.cloud,
        public_html = _ref.public_html;

    _classCallCheck(this, VendorAdapter);

    this.config = config;
    this.cloud = cloud;
    this.public_html = public_html;
    this.cli_repository = 'ParsePlatform/parse-cli';
  }

  _createClass(VendorAdapter, [{
    key: 'getCliLatestVersion',
    value: function getCliLatestVersion() {
      var _this = this;

      return new Promise(function (resolve, reject) {
        _https2.default.get({
          host: "api.github.com",
          path: '/repos/' + _this.cli_repository + '/releases',
          headers: {
            "User-Agent": "back4app/parse-cli-server"
          }
        }, function (response) {
          var data = '';
          response.on('data', function (chunk) {
            data += chunk;
          });
          response.on('end', function () {
            var releases = JSON.parse(data),
                latest = releases[0];
            resolve(latest.tag_name.slice(8));
          });
        }).on('error', function (error) {
          console.error(error);
          throw error;
        });
      });
    }
  }, {
    key: 'getAccountKey',
    value: function getAccountKey(email, password) {
      return Promise.resolve(password == this.config.masterKey ? 'ACCOUNT_KEY_' + email : null);
    }
  }, {
    key: 'getEmail',
    value: function getEmail(accountKey) {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        if (accountKey == _this2.config.masterKey) {
          var email = _this2.config.appName + "@example.com";
          resolve(email);
        } else {
          reject("not found");
        }
      });
    }
  }, {
    key: 'getJsVersions',
    value: function getJsVersions(appId) {
      return new Promise(function (resolve, reject) {
        resolve([_package.version]);
      });
    }
  }, {
    key: 'getApps',
    value: function getApps(accountKey) {
      return this.getApp(accountKey, this.config.appId).then(function (parseApp) {
        return [parseApp];
      });
    }
  }, {
    key: 'getApp',
    value: function getApp(accountKey, appId) {
      var _this3 = this;

      return new Promise(function (resolve, reject) {
        resolve({
          appName: _this3.config.appName,
          dashboardURL: null, // TODO
          applicationId: appId,
          clientKey: _this3.config.clientKey,
          javascriptKey: _this3.config.javascriptKey,
          windowsKey: _this3.config.dotNetKey,
          webhookKey: _this3.config.webhookKey,
          restKey: _this3.config.restAPIKey,
          masterKey: _this3.config.masterKey,
          clientPushEnabled: false, // TODO
          clientClassCreationEnabled: _this3.config.allowClientClassCreation,
          requireRevocableSessions: false, // TODO
          revokeSessionOnPasswordChange: _this3.config.revokeSessionOnPasswordReset
        });
      });
    }
  }, {
    key: 'createApp',
    value: function createApp(accountKey, appName) {
      return new Promise(function (resolve, reject) {
        reject("Create app is not implemented. Parse Server does not support multiple apps.");
      });
    }
  }, {
    key: 'collect',
    value: function collect(appId, deployInfo, folder, filename, data) {
      var deployPath = _path2.default.join(_os2.default.tmpdir(), 'parse-cli-server', deployInfo.releaseName, folder);
      return _fsExtra2.default.ensureDir(deployPath, function (err) {
        if (err) {
          throw err;
        }
        var filePath = _path2.default.join(deployPath, filename);
        return _fsExtra2.default.outputFile(filePath, data, function (err) {
          if (err) {
            return console.log(err);
          }
        });
      });
    }
  }, {
    key: '_copy',
    value: function _copy(from, to) {
      return new Promise(function (resolve, reject) {
        if (!to) {
          resolve();
          return;
        }

        _fsExtra2.default.copy(from, to, function (err) {
          if (err) {
            throw err;
          }
          resolve();
        });
      });
    }
  }, {
    key: 'publish',
    value: function publish(appId, deployInfo) {
      var _this4 = this;

      var deployPath = _path2.default.join(_os2.default.tmpdir(), 'parse-cli-server', deployInfo.releaseName);
      return this._copy(_path2.default.join(deployPath, 'cloud'), _path2.default.dirname(this.cloud)).then(function () {
        return _this4._copy(_path2.default.join(deployPath, 'public'), _this4.public_html);
      }).then(function () {
        return _this4._reloadCloudCode();
      }).then(function () {
        console.log("Published!");
      });
    }
  }, {
    key: '_reloadCloudCode',
    value: function _reloadCloudCode() {
      triggers._unregisterAll();

      // cleaning require cache
      Object.keys(require.cache).forEach(function (key) {
        delete require.cache[key];
      });

      var mainFile = _path2.default.resolve(process.cwd(), this.cloud);
      require(mainFile);
    }

    // optional collectionName customization

  }, {
    key: 'getCollectionName',
    value: function getCollectionName(appId, collectionName) {
      return collectionName;
    }

    // optional basePath customization

  }, {
    key: 'getBasePath',
    value: function getBasePath(appId, basePath) {
      return basePath;
    }
  }]);

  return VendorAdapter;
}();

exports.default = VendorAdapter;
exports.VendorAdapter = VendorAdapter;